import axios from "axios";
import Company from "../models/company-model.js";
import User from "../models/user-model.js";

export const createCompanybasic = async (data) => {
  try {
    const {
      appicationURL, // Corrected property name
      applicationName, // Corrected property name
      ipAddress,
      applicationDescription, // Corrected property name
      userid,
    } = data;

    console.log({ userid });

    // Check if the user with the provided userid exists
    const existingUser = await User.findOne({ _id: userid });

    if (!existingUser) {
      throw new Error("User with this ID does not exist.");
    }

    const existingCompany = await Company.findOne({ userid });

    if (existingCompany) {
      // If a company with the same userid already exists, return an error message
      throw Error("User already has a company record.");
    }

    const newCompany = new Company({
      userid: userid,
      basicinfo: {
        appUrl: appicationURL,
        appName: applicationName,
        ipAddress: ipAddress,
        appDesc: applicationDescription,
      },
    });

    const createdCompany = await newCompany.save();

    // Update the user's companyId field with the ID of the newly created company
    existingUser.companyId = createdCompany._id;
    await existingUser.save();

    return createdCompany;
  } catch (error) {
    console.log(error);
    throw error;
  }
};

export const createTechnicalInformation = async (data) => {
  try {
    // Find the company by ID
    const company = await Company.findById(data.companyid);

    if (!company) {
      throw new Error(`Company with ID ${data.companyid} not found`);
    }

    // Update the technical information fields
    company.techinfo = {
      financialvalue: data.financialvalue || company.techinfo.financialvalue,
      frontend: data.frontend || company.techinfo.frontend,
      backend: data.backend || company.techinfo.backend,
      database: data.database || company.techinfo.database,
      application: data.application || company.techinfo.application,
      others: data.others || company.techinfo.others,
      webAppType: data.webAppType || company.techinfo.webAppType,
    };

    // Save the updated company
    const updatedCompany = await company.save();
    return updatedCompany;
  } catch (error) {
    console.log(error);
    throw error;
  }
};

export const createContactInformation = async (data) => {
  try {
    // Find the company by ID
    const company = await Company.findById(data.companyid);

    if (!company) {
      throw new Error(`Company with ID ${data.companyid} not found`);
    }

    // Update the technical information fields
    company.contactinfo = {
      companyName: data.companyName || company.companyName,
      contactNumber: data.contactNumber || company.contactNumber,
      address: data.address || company.address,
    };

    // Save the updated company
    const updatedCompany = await company.save();
    return updatedCompany;
  } catch (error) {
    console.log(error);
    throw error;
  }
};

export const getCompanyById = async (id) => {
  try {
    const company = await Company.findById(id);
    return company;
  } catch (error) {
    console.log(error);
    throw error;
  }
};

export const getVulnerabilities = async (id) => {
  try {
    const company = await Company.findById(id);

    if (!company) {
      throw new Error("Company not found");
    }

    const { frontend, backend, database, others, application } =
      company.techinfo;

    const techinfoArray = [frontend, backend, database, others, application];

    const results = [];

    const config = {
      headers: {
        apikey: "57207422-e8b1-4e84-8ba3-7bc3029a14f8",
      },
    };

    for (const tech of techinfoArray) {
      const response = await axios.get(
        `https://services.nvd.nist.gov/rest/json/cves/2.0/?keywordSearch=${tech}&lastModStartDate=2023-08-19T13:00:00.000%2B01:00&lastModEndDate=2023-09-03T13:36:00.000%2B01:00`,
        config
      );

      const vulnerabilities = response.data.vulnerabilities || [];

      const vulnerabilitiesData = vulnerabilities.map((vuln) => {
        const baseSeverity =
          vuln.cve?.metrics?.cvssMetricV31?.[0]?.cvssData?.baseSeverity ||
          "none";
        return {
          id: vuln.cve ? vuln.cve.id : "none",
          description: vuln.cve
            ? vuln.cve.descriptions.find((desc) => desc.lang === "en")?.value ||
              "none"
            : "none",
          baseSeverity,
        };
      });

      if (vulnerabilitiesData.length > 0) {
        // If there are vulnerabilities, add the tech name and baseSeverity to the results
        results.push({ tech, vulnerabilities: vulnerabilitiesData });
      } else {
        // If there are no vulnerabilities, add the tech name with an empty array
        results.push({ tech, vulnerabilities: [] });
      }
    }

    return results;
  } catch (error) {
    console.log(error);
    throw error;
  }
};

// Function to get vulnerabilities by sorting date in the last 7 days
export const getVulnerabilitiesByLastSevenDays = async (id) => {
  try {
    const company = await Company.findById(id);

    if (!company) {
      throw new Error("Company not found");
    }

    const { frontend, backend, database, others, application } =
      company.techinfo;

    const techinfoArray = [frontend, backend, database, others, application];

    const results = [];

    // Calculate the start and end dates for the last 7 days
    const endDate = new Date(); // Today's date
    const startDate = new Date();
    startDate.setDate(endDate.getDate() - 7); // 7 days ago

    const config = {
      headers: {
        apikey: "57207422-e8b1-4e84-8ba3-7bc3029a14f8",
      },
    };

    for (const tech of techinfoArray) {
      const response = await axios.get(
        `https://services.nvd.nist.gov/rest/json/cves/2.0/?keywordSearch=${tech}&lastModStartDate=${startDate.toISOString()}&lastModEndDate=${endDate.toISOString()}`,
        config
      );

      const vulnerabilities = response.data.vulnerabilities || [];

      const vulnerabilitiesData = vulnerabilities.map((vuln) => {
        const baseSeverity =
          vuln.cve?.metrics?.cvssMetricV31?.[0]?.cvssData?.baseSeverity ||
          "none";

        return {
          id: vuln.cve ? vuln.cve.id : "none",
          description: vuln.cve
            ? vuln.cve.descriptions.find((desc) => desc.lang === "en")?.value ||
              "none"
            : "none",
          baseSeverity,
        };
      });

      if (vulnerabilitiesData.length > 0) {
        // If there are vulnerabilities, add the tech name and baseSeverity to the results
        results.push({
          tech,
          vulnerabilities: vulnerabilitiesData,
          count: vulnerabilitiesData.length,
        });
      } else {
        // If there are no vulnerabilities, add the tech name with an empty array
        results.push({
          tech,
          vulnerabilities: [],
          count: vulnerabilitiesData.length,
        });
      }
    }

    return results;
  } catch (error) {
    console.log(error);
    throw error;
  }
};

// Function to get vulnerabilities for a specific day
export const getVulnerabilitiesForSpecificDay = async (id, specificDate) => {
  try {
    const company = await Company.findById(id);

    if (!company) {
      throw new Error("Company not found");
    }

    const { frontend, backend, database, others, application } =
      company.techinfo;

    const techinfoArray = [frontend, backend, database, others, application];

    const results = [];

    // Calculate the start and end dates for the last 7 days
    const endDate = new Date(); // Today's date
    const startDate = new Date();
    startDate.setDate(endDate.getDate() - 1);

    const config = {
      headers: {
        apikey: "57207422-e8b1-4e84-8ba3-7bc3029a14f8",
      },
    };



    for (const tech of techinfoArray) {
      const response = await axios.get(
        `https://services.nvd.nist.gov/rest/json/cves/2.0/?keywordSearch=${tech}&lastModStartDate=${startDate.toISOString()}&lastModEndDate=${endDate.toISOString()}`,
        config
      );

      const vulnerabilities = response.data.vulnerabilities || [];

      const vulnerabilitiesData = await Promise.all(
        vulnerabilities.map(async (vuln) => {
          const baseSeverity =
            vuln.cve?.metrics?.cvssMetricV31?.[0]?.cvssData?.baseSeverity ||
            "none";

          const formData = new FormData();
          formData.append(
            "severity",
            vuln.cve
              ? vuln.cve.descriptions.find((desc) => desc.lang === "en")
                  ?.value || "none"
              : "none"
          );

          const serres = await axios.post(
            "http://127.0.0.1:5000/severity",
            formData
          );

          return {
            id: vuln.cve ? vuln.cve.id : "none",
            description: vuln.cve
              ? vuln.cve.descriptions.find((desc) => desc.lang === "en")
                  ?.value || "none"
              : "none",
            baseSeverity,
            secondApiseverity: serres.data.result, // Modify this as needed
          };
        })
      );

      if (vulnerabilitiesData.length > 0) {
        // If there are vulnerabilities, add the tech name and baseSeverity to the results
        results.push({
          tech,
          vulnerabilities: vulnerabilitiesData,
          count: vulnerabilitiesData.length,
        });
      } else {
        // If there are no vulnerabilities, add the tech name with an empty array
        results.push({
          tech,
          vulnerabilities: [],
          count: vulnerabilitiesData.length,
        });
      }
    }

    return results;
  } catch (error) {
    console.log(error);
    throw error;
  }
};
