import Log from "../models/log-model.js";
import OTP from "../models/otp-model.js";
import User from "../models/user-model.js";
import { generateOTP } from "../utils/generateOTP.js";
import { hashData, verifyHashedData } from "../utils/hash-data.js";
import { sendEmail } from "../utils/sendEmail.js";

export const verifyOTP = async ({ email, otp }) => {
  try {
    if (!(email && otp)) {
      throw Error("Provide values for email, otp");
    }
    // ensure otp record exists
    const matchedOTPRecord = await OTP.findOne({
      email,
    });

    if (!matchedOTPRecord) {
      throw Error("NO OTP Records Found");
    }

    const { expiresAt } = matchedOTPRecord;
    // checking for expired code
    if (expiresAt < Date.now()) {
      await OTP.deleteOne({ email });
      throw Error("Code has expired. Request for a new one.");
    }
    // not expired yet, verify value

    const hashedOtp = matchedOTPRecord.otp;
    const validOTP = await verifyHashedData(otp, hashedOtp);

    if (validOTP) {
      await OTP.deleteOne({ email });
    }

    // Mark the user as verified

    if (validOTP) {
      const userByEmail = await User.findOne({ email: email });
      console.log({ userByEmail });
      await User.findOneAndUpdate(
        { email: email },
        { $set: { isVerified: true } },
        { new: true } // To return the updated user document
      );
    }

    return otp;
  } catch (error) {
    throw error;
  }
};

export const deleteOTP = async (email) => {
  try {
    await OTP.deleteOne({ email });
  } catch (error) {
    throw error;
  }
};

export const sendOTP = async ({ email, subject, message, duration = 1 }) => {
  try {
    if (!(email && subject && message)) {
      throw Error("Provide values for email, subject,message");
    }

    // clear any old record
    await OTP.deleteOne({ email });

    await User.findOneAndUpdate(
      { email: email },
      { $set: { isVerified: false } },
      { new: true } // To return the updated user document
    );

    //Delete enerate pin
    const generatedOTP = await generateOTP();
    console.log({ generateOTP });

    // send email
    const mailOptions = {
      from: process.env.AUTH_EMAIL,
      to: email,
      subject,
      html: `<h3>Webguardian Email Verification</h3><p>${message}</p><p style="color:tomato;
    font-size:25px; letter-spacing:2px;"><b>${generatedOTP}</b></p><p>This code <b>expires in ${duration} hour(s)</b>.</p>`,
    };
    await sendEmail(mailOptions);

    console.log("start");
    `1`;
    const hashedOTP = await hashData(generatedOTP);
    console.log("end");
    const newOTP = await new OTP({
      email,
      otp: hashedOTP,
      createdAt: Date.now(),
      expiresAt: Date.now() + 3600000 * +duration,
    });
    const createdOTPRecord = await newOTP.save();
    return createdOTPRecord;
  } catch (error) {
    console.log(error);
  }
};

// get log data

export const getAllLogData = async () => {
  try {
    const logData = await Log.find();
    return logData;
  } catch (error) {
    throw error;
  }
};
